@@ -20,13 +20,14 @@
  */
 
 #include <goffice/goffice-config.h>
+#include <goffice/app/go-doc.h>
 #include <goffice/graph/gog-graph-impl.h>
 #include <goffice/graph/gog-chart-impl.h>
 #include <goffice/graph/gog-renderer.h>
-#include <goffice/graph/gog-renderer-cairo.h>
 #include <goffice/graph/gog-style.h>
 #include <goffice/graph/gog-theme.h>
 #include <goffice/data/go-data.h>
+#include <goffice/utils/go-persist.h>
 #include <goffice/utils/go-units.h>
 
 #include <gsf/gsf-impl-utils.h>
@@ -49,7 +50,8 @@ enum {
 	GRAPH_PROP_THEME,
 	GRAPH_PROP_THEME_NAME,
 	GRAPH_PROP_WIDTH,
-	GRAPH_PROP_HEIGHT
+	GRAPH_PROP_HEIGHT,
+	GRAPH_PROP_DOCUMENT
 };
 
 enum {
@@ -86,6 +88,16 @@ gog_graph_set_property (GObject *obj, guint param_id,
 		gog_graph_set_size (graph, graph->width, 
 				    g_value_get_double (value));
 		break;
+	case GRAPH_PROP_DOCUMENT: {
+		GObject *object = g_value_get_object (value);
+		if ((GODoc *) object == graph->doc)
+			break;
+		if (IS_GO_DOC (obj)) {
+			graph->doc = (GODoc *) object;
+			gog_object_document_changed (GOG_OBJECT (graph), graph->doc);
+		}
+		break;
+	}
 
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 return; /* NOTE : RETURN */
@@ -111,6 +123,9 @@ gog_graph_get_property (GObject *obj, guint param_id,
 	case GRAPH_PROP_HEIGHT:
 		g_value_set_double (value, graph->height);
 		break;
+	case GRAPH_PROP_DOCUMENT:
+		g_value_set_object (value, graph->doc);
+		break;
 
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 break;
@@ -320,19 +335,25 @@ gog_graph_class_init (GogGraphClass *klass)
 			_("Theme name"),
 			_("The name of the theme for elements of the graph"),
 			"default", 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, GRAPH_PROP_WIDTH,
 		g_param_spec_double ("width-pts", 
 			_("Width"),
 			_("Logical graph width, in points"),
 			0.0, G_MAXDOUBLE, GOG_GRAPH_DEFAULT_WIDTH,
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, GRAPH_PROP_HEIGHT,
 		g_param_spec_double ("height-pts", 
 			_("Height"),
 			_("Logical graph heigth, in points"),
 			0.0, G_MAXDOUBLE, GOG_GRAPH_DEFAULT_HEIGHT,
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
+	g_object_class_install_property (gobject_klass, GRAPH_PROP_DOCUMENT,
+		g_param_spec_object ("document", 
+			_("Document"),
+			_("the document for this graph"),
+			GO_DOC_TYPE, 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE));
 }
 
 static void
@@ -367,7 +388,9 @@ GSF_CLASS (GogGraph, gog_graph,
  *
  * Check the layout of the chart grid and ensure that there are no empty
  * cols or rows, and resize as necessary
- */
+ *
+ * Returns: %TRUE if anything changed.
+ **/
 gboolean
 gog_graph_validate_chart_layout (GogGraph *graph)
 {
@@ -455,12 +478,13 @@ gog_graph_num_rows (GogGraph const *graph)
  * gog_graph_dup :
  * @graph : #GogGraph
  *
- * A convenience wrapper to make a deep copy of @graph.
+ * Returns: a deep copy of @graph.
  **/
 GogGraph *
 gog_graph_dup (GogGraph const *graph)
 {
 	GogObject *res = gog_object_dup (GOG_OBJECT (graph), NULL, NULL);
+	GOG_GRAPH (res)->doc = graph->doc;
 	return GOG_GRAPH (res);
 }
 
@@ -524,9 +548,9 @@ gog_graph_get_data (GogGraph const *graph)
  * @graph : #GogGraph
  * @dat : #GOData
  *
- * If @dat or something equivalent to it already exists in the graph use that.
- * Otherwaise use @dat.  Adds a gobject ref to the target and increments a
- * count of the number of refs made from this #GogGraph.
+ * Returns: @dat or something equivalent to it that already exists in the graph.
+ * 	Otherwise use @dat.  Adds a gobject ref to the target and increments a
+ *	 count of the number of refs made from this #GogGraph.
  **/
 GOData *
 gog_graph_ref_data (GogGraph *graph, GOData *dat)
@@ -794,11 +818,8 @@ gog_graph_view_render  (GogView *view, GogViewAllocation const *bbox)
 
 	gview_parent_klass->render (view, bbox);
 
-	if (gview->selected_view != NULL) {
-		gog_renderer_push_selection_style (view->renderer);
+	if (gview->selected_view != NULL)
 		gog_view_render_toolkit (gview->selected_view);
-		gog_renderer_pop_style (view->renderer);
-	}
 }
 
 static void
@@ -897,9 +918,11 @@ update_action (GogGraphView *view, GogTool *tool, double x, double y)
  * gog_graph_view_handle_event:
  * @gview : #GogGraphView
  * @event : #GdkEvent
+ * @x_offset : in pixels
+ * @y_offset : in pixels
  *
  * Handle events.
- */
+ **/
 void
 gog_graph_view_handle_event (GogGraphView *view, GdkEvent *event, 
 			     double x_offset, double y_offset)
@@ -974,7 +997,7 @@ GogView *
 gog_graph_view_get_selection (GogGraphView *gview)
 {
 	g_return_val_if_fail (IS_GOG_GRAPH_VIEW (gview), NULL);
-	
+
 	return gview->selected_view;
 }
 
@@ -990,7 +1013,7 @@ void
 gog_graph_view_set_selection (GogGraphView *gview, GogObject *gobj)
 {
 	GogView *view;
-	
+
 	g_return_if_fail (IS_GOG_GRAPH_VIEW (gview));
 	g_return_if_fail (IS_GOG_OBJECT (gobj));
 
@@ -1023,6 +1046,9 @@ GSList *
 gog_graph_get_supported_image_formats (void)
 {
 	static GOImageFormat supported_formats[] = {
+#ifdef HAVE_CAIRO_PS_SURFACE_SET_EPS
+		GO_IMAGE_FORMAT_EPS,
+#endif
 #ifdef GOG_RENDERER_CAIRO_WITH_PS
 		GO_IMAGE_FORMAT_PS,
 #endif
@@ -1063,27 +1089,34 @@ gog_graph_export_image (GogGraph *graph, GOImageFormat format, GsfOutput *output
 {
 	GogRenderer *renderer;
 	gboolean result;
-		
+
 	g_return_val_if_fail (IS_GOG_GRAPH (graph), FALSE);
 	g_return_val_if_fail (format != GO_IMAGE_FORMAT_UNKNOWN, FALSE);
-	
-	renderer = gog_renderer_new_for_format (graph, format);
-	g_return_val_if_fail (renderer != NULL, FALSE);
 
+	renderer = gog_renderer_new (graph);
 	result = gog_renderer_export_image (renderer, format, output, x_dpi, y_dpi);
 	g_object_unref (renderer);
 
 	return result;
 }
 
-void gog_graph_render_to_cairo (GogGraph *graph, gpointer data, double w, double h)
+/**
+ * gog_graph_render_to_cairo:
+ * @graph: a #GogGraph
+ * @cairo: a cairo context
+ * @w: width
+ * @h: height
+ *
+ * Renders a graph using @cairo. @w and @h are the requested width an height of the rendered graph, in the current @cairo coordinate space. This function is not suited for multiple rendering of the same graph. gog_renderer_render_to cairo or gog_renderer_update/gog_renderer_get_cairo_surface should be used instead.
+ **/
+
+void gog_graph_render_to_cairo (GogGraph *graph, cairo_t *cairo, double w, double h)
 {
-	GogRendererCairo *rend = GOG_RENDERER_CAIRO (g_object_new (
-		GOG_RENDERER_CAIRO_TYPE,
-		"model", graph,
-		"cairo", data,
-		"is-vector", FALSE,
-		NULL));
-	gog_renderer_cairo_update (rend, w, h, 1.);
-	g_object_unref (rend);
+	GogRenderer *renderer;
+
+	g_return_if_fail (IS_GOG_GRAPH (graph));
+
+	renderer = gog_renderer_new (graph);
+	gog_renderer_render_to_cairo (renderer, cairo, w, h);
+	g_object_unref (renderer);
 }
