@@ -23,6 +23,7 @@
 #include <goffice/goffice-config.h>
 #include <goffice/goffice-priv.h>
 
+#include <goffice/app/go-doc-control.h>
 #include <goffice/graph/gog-guru.h>
 #include <goffice/graph/gog-object.h>
 #include <goffice/graph/gog-graph.h>
@@ -222,11 +223,6 @@ graph_typeselect_minor (GraphGuruTypeSelector *typesel, FooCanvasItem *item)
 	gog_object_add_by_name (GOG_OBJECT (s->chart),
 		"Plot", GOG_OBJECT (s->plot = plot));
 	gog_plot_guru_helper (plot);
-#if 0
-	if (s->original_plot != NULL &&
-	    !gog_plot_make_similar (s->plot, s->original_plot))
-		return;
-#endif
 
 	if (s->dalloc != NULL)
 		gog_data_allocator_allocate (s->dalloc, s->plot);
@@ -756,13 +752,12 @@ populate_graph_item_list (GogObject *obj, GogObject *select, GraphGuruState *s,
 	if (insert) {
 		GogObject *gparent = gog_object_get_parent (obj);
 		gint i = g_slist_index (gparent->children, obj);
-		if (i > 0) {
-			GtkTreeIter sibling;
-			gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (s->prop_model),
-				&sibling, parent, i-1);
-			gtk_tree_store_insert_after (s->prop_model, &iter,
-				parent, &sibling);
-		} else
+		GtkTreeIter sibling;
+		if (i > 0 && gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (s->prop_model),
+													&sibling, parent, i-1))
+				gtk_tree_store_insert_after (s->prop_model, &iter,
+					parent, &sibling);
+		else
 			gtk_tree_store_prepend (s->prop_model, &iter, parent);
 	} else
 		gtk_tree_store_append (s->prop_model, &iter, parent);
@@ -1311,6 +1306,14 @@ gog_guru (GogGraph *graph, GogDataAllocator *dalloc,
 		state->graph = g_object_new (GOG_GRAPH_TYPE, NULL);
 		state->chart = GOG_CHART (gog_object_add_by_name (
 				GOG_OBJECT (state->graph), "Chart", NULL));
+		if (IS_GO_DOC_CONTROL (dalloc))
+			g_object_set (state->graph,
+						  "document", go_doc_control_get_doc (GO_DOC_CONTROL (dalloc)),
+						  NULL);
+		else if (IS_GO_DOC_CONTROL (cc))
+			g_object_set (state->graph,
+						  "document", go_doc_control_get_doc (GO_DOC_CONTROL (cc)),
+						  NULL);
 	}
 
 	if (state->graph == NULL || graph_guru_init (state)) {
