@@ -2,7 +2,8 @@
 /*
  * gog-radar.c
  *
- * Copyright (C) 2004 Michael Devine (mdevine@cs.stanford.edu)
+ * Copyright (C) 2004 Michael Devine <mdevine@cs.stanford.edu>
+ * Copyright (C) 2007 Emmanuel Pacaud <emmanuel.pacaud@lapp.in2p3.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -23,15 +24,17 @@
 #include "gog-radar.h"
 #include <goffice/graph/gog-axis.h>
 #include <goffice/graph/gog-chart.h>
+#include <goffice/graph/gog-chart-map.h>
 #include <goffice/graph/gog-grid-line.h>
 #include <goffice/graph/gog-view.h>
 #include <goffice/graph/gog-renderer.h>
 #include <goffice/graph/gog-theme.h>
 #include <goffice/graph/gog-style.h>
 #include <goffice/data/go-data.h>
+#include <goffice/math/go-math.h>
 #include <goffice/utils/go-color.h>
 #include <goffice/utils/go-marker.h>
-#include <goffice/utils/go-math.h>
+#include <goffice/utils/go-persist.h>
 #include <goffice/app/module-plugin-defs.h>
 
 #include <glib/gi18n-lib.h>
@@ -39,27 +42,45 @@
 
 #include <string.h>
 
+#ifdef GOFFICE_WITH_GTK
+#include <goffice/gtk/goffice-gtk.h>
+#include <gtk/gtktogglebutton.h>
+#include <gtk/gtkcombobox.h>
+#endif
+
 typedef struct {
 	GogPlotClass	base;
 } GogRTPlotClass;
 
 enum {
 	PLOT_PROP_0,
-	PLOT_PROP_DEFAULT_STYLE_HAS_MARKERS
+	PLOT_PROP_DEFAULT_STYLE_HAS_MARKERS,
+	PLOT_PROP_DEFAULT_STYLE_HAS_FILL
 };
 
 GOFFICE_PLUGIN_MODULE_HEADER;
 
 typedef struct {
-	GogSeries base;
+	GogSeries 	   base;
+	GogObject 	  *radial_drop_lines;
 } GogRTSeries;
+
+typedef GogRTSeries GogPolarSeries;
+
 typedef GogSeriesClass GogRTSeriesClass;
+typedef GogRTSeriesClass GogPolarSeriesClass;
+
+enum {
+	SERIES_PROP_0,
+	SERIES_PROP_FILL_TYPE
+};
 
 #define GOG_RT_SERIES_TYPE	(gog_rt_series_get_type ())
 #define GOG_RT_SERIES(o)	(G_TYPE_CHECK_INSTANCE_CAST ((o), GOG_RT_SERIES_TYPE, GogRTSeries))
 #define GOG_IS_RT_SERIES(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), GOG_RT_SERIES_TYPE))
 
 static GType gog_rt_series_get_type (void);
+static GType gog_polar_series_get_type (void);
 static GType gog_rt_view_get_type (void);
 
 /*-----------------------------------------------------------------------------
@@ -84,6 +105,9 @@ gog_rt_plot_set_property (GObject *obj, guint param_id,
 	case PLOT_PROP_DEFAULT_STYLE_HAS_MARKERS:
 		rt->default_style_has_markers = g_value_get_boolean (value);
 		break;
+	case PLOT_PROP_DEFAULT_STYLE_HAS_FILL:
+		rt->default_style_has_fill = g_value_get_boolean (value);
+		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 return; /* NOTE : RETURN */
 	}
@@ -103,6 +127,9 @@ gog_rt_plot_get_property (GObject *obj, guint param_id,
 	case PLOT_PROP_DEFAULT_STYLE_HAS_MARKERS:
 		g_value_set_boolean (value, rt->default_style_has_markers);
 		break;
+	case PLOT_PROP_DEFAULT_STYLE_HAS_FILL:
+		g_value_set_boolean (value, rt->default_style_has_fill);
+		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 break;
 	}
@@ -174,13 +201,20 @@ gog_rt_plot_class_init (GogPlotClass *gog_plot_klass)
 	gog_object_klass->update	= gog_rt_plot_update;
 	gog_object_klass->view_type	= gog_rt_view_get_type ();
 
-	g_object_class_install_property (gobject_klass, 
-					 PLOT_PROP_DEFAULT_STYLE_HAS_MARKERS,
-		g_param_spec_boolean ("default-style-has-markers", 
+	g_object_class_install_property (gobject_klass,
+		PLOT_PROP_DEFAULT_STYLE_HAS_MARKERS,
+		g_param_spec_boolean ("default-style-has-markers",
 			_("Default markers"),
 			_("Should the default style of a series include markers"),
-			FALSE, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			FALSE,
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
+	g_object_class_install_property (gobject_klass,
+		PLOT_PROP_DEFAULT_STYLE_HAS_FILL,
+		g_param_spec_boolean ("default-style-has-fill",
+			_("Default fill"),
+			_("Should the default style of a series include fill"),
+			FALSE,
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 
 	/* Fill in GogPlotClass methods */
 	gog_plot_klass->desc.num_series_min = 1;
@@ -196,6 +230,7 @@ gog_rt_plot_init (GogRTPlot *rt)
 {
 	rt->base.vary_style_by_element = FALSE;
 	rt->default_style_has_markers = FALSE;
+	rt->default_style_has_fill = FALSE;
 	rt->num_elements = 0;
 }
 
@@ -308,11 +343,12 @@ gog_radar_area_plot_class_init (GogObjectClass *gog_klass)
 }
 
 static void
-gog_radar_area_plot_init (GogPlot *plot) 
+gog_radar_area_plot_init (GogPlot *plot)
 {
+	GOG_RT_PLOT(plot)->default_style_has_fill = TRUE;
 	plot->render_before_axes = TRUE;
 }
-	
+
 GSF_DYNAMIC_CLASS (GogRadarAreaPlot, gog_radar_area_plot,
 	gog_radar_area_plot_class_init, gog_radar_area_plot_init,
 	GOG_RADAR_PLOT_TYPE)
@@ -351,8 +387,8 @@ gog_polar_plot_axis_get_bounds (GogPlot *plot, GogAxisType axis,
 	default:
 		g_warning("[GogPolarPlot::axis_set_bounds] bad axis (%i)", axis);
 		break;
-	} 
-	
+	}
+
 	return NULL;
 }
 
@@ -373,9 +409,13 @@ gog_polar_plot_class_init (GogPlotClass *gog_plot_klass)
 		};
 		gog_plot_klass->desc.series.dim = dimensions;
 		gog_plot_klass->desc.series.num_dim = G_N_ELEMENTS (dimensions);
-		gog_plot_klass->desc.series.style_fields = GOG_STYLE_LINE | GOG_STYLE_MARKER;
+		gog_plot_klass->desc.series.style_fields = GOG_STYLE_LINE 
+			| GOG_STYLE_FILL 
+			| GOG_STYLE_MARKER
+			| GOG_STYLE_INTERPOLATION;
 	}
 
+	gog_plot_klass->series_type = gog_polar_series_get_type();
 	gog_plot_klass->axis_get_bounds	= gog_polar_plot_axis_get_bounds;
 }
 
@@ -398,12 +438,10 @@ gog_rt_view_render (GogView *view, GogViewAllocation const *bbox)
 	GogChartMap *chart_map;
 	GogChartMapPolarData *parms;
 	GogViewAllocation const *area;
+	GOPath *next_path = NULL;
 	GSList   *ptr;
-	ArtVpath *path, *clip_path;
-	ArtBpath *bpath = NULL;
-	double th0, theta_min, theta_max, theta;
+	double th0, theta_min, theta_max, theta = 0;
 	double rho_min, rho_max, rho;
-	gboolean const is_area = GOG_IS_PLOT_RADAR_AREA (model);
 	gboolean const is_polar = GOG_IS_PLOT_POLAR (model);
 
 	r_axis = GOG_PLOT (model)->axis[GOG_AXIS_RADIAL];
@@ -419,7 +457,7 @@ gog_rt_view_render (GogView *view, GogViewAllocation const *bbox)
 	c_map = gog_chart_map_get_axis_map (chart_map, 0);
 	r_map = gog_chart_map_get_axis_map (chart_map, 1);
 	parms = gog_chart_map_get_polar_parms (chart_map);
-	
+
 	gog_axis_map_get_bounds (c_map, &theta_min, &theta_max);
 	th0 = theta_min;
 	gog_axis_map_get_bounds (r_map, &rho_min, &rho_max);
@@ -433,83 +471,138 @@ gog_rt_view_render (GogView *view, GogViewAllocation const *bbox)
 		theta_max = theta;
 	}
 
-	path = g_alloca ((model->num_elements + 2) * sizeof (ArtVpath));
 	for (ptr = model->base.series; ptr != NULL; ptr = ptr->next) {
 
 		GogRTSeries *series = GOG_RT_SERIES (ptr->data);
 		GogStyle *style;
-		gboolean closed_shape;
+		GOPath *path;
 		unsigned count;
-		double   *r_vals, *c_vals = NULL;
+		double *r_vals, *c_vals;
+		double x, y;
 
-		if (!gog_series_is_valid (GOG_SERIES (series))) 
+		if (!gog_series_is_valid (GOG_SERIES (series)))
 			continue;
 
 		style = GOG_STYLED_OBJECT (series)->style;
 
 		gog_renderer_push_style (view->renderer, style);
 
-		closed_shape = (series->base.num_elements == model->num_elements);
 		r_vals = go_data_vector_get_values (GO_DATA_VECTOR (series->base.values[1].data));
+		c_vals = is_polar ?  go_data_vector_get_values (GO_DATA_VECTOR (series->base.values[0].data)) : NULL;
+
+		if (is_polar) {
+			GOPath *clip_path;
+
+			clip_path = go_path_new ();
+			go_path_ring_wedge (clip_path, parms->cx, parms->cy, parms->rx, parms->ry,
+					    0.0, 0.0, -parms->th0, -parms->th1);
+			gog_renderer_push_clip (view->renderer, clip_path);
+			go_path_free (clip_path);
+		}
+
+		if (next_path != NULL)
+			path = next_path;
+		else
+			path = gog_chart_map_make_path (chart_map, c_vals,r_vals, series->base.num_elements,
+							series->base.interpolation);
+
+		next_path = NULL;
+
+		if (!is_polar) {
+			go_path_close (path);
+			gog_renderer_fill_serie (view->renderer, path, NULL);
+		} else {
+			if (series->base.fill_type != GOG_SERIES_FILL_TYPE_NEXT) {
+				GOPath *close_path;
+
+				close_path = gog_chart_map_make_close_path (chart_map, c_vals, r_vals,
+									    series->base.num_elements,
+									    series->base.fill_type);
+				gog_renderer_fill_serie (view->renderer, path, close_path);
+				if (close_path != NULL)
+					go_path_free (close_path);
+			} else {
+				if (ptr->next != NULL) {
+					GogRTSeries *next_series;
+
+					next_series = ptr->next->data;
+					if (gog_series_is_valid (GOG_SERIES (next_series))) {
+						GogStyle *next_style;
+						const double *next_x_vals, *next_y_vals;
+						unsigned int next_n_points;
+
+						next_n_points = gog_series_get_xy_data
+							(GOG_SERIES (next_series),
+							 &next_x_vals, &next_y_vals);
+						next_style = gog_styled_object_get_style
+							(GOG_STYLED_OBJECT (next_series));
+
+						next_path = gog_chart_map_make_path
+							(chart_map, next_x_vals, next_y_vals,
+							 next_n_points, next_series->base.interpolation);
+
+					}
+				}
+				gog_renderer_fill_serie (view->renderer, path, next_path);
+			}
+		}
+
+		if (series->radial_drop_lines != NULL) {
+			GOPath *drop_path;
+			double theta, rho;
+			unsigned int i;
+
+			gog_renderer_push_style (view->renderer,
+				gog_styled_object_get_style (GOG_STYLED_OBJECT (series->radial_drop_lines)));
+			drop_path = go_path_new ();
+			for (i = 0; i < series->base.num_elements; i++) {
+				gog_chart_map_2D_to_view (chart_map, ((c_vals != NULL) ? c_vals[i] : i+1),
+							  r_vals[i], &theta, &rho);
+				if (   go_finite (theta) 
+				    && go_finite (rho) 
+				    && rho > rho_min) {
+					go_path_move_to (drop_path, parms->cx, parms->cy);
+					go_path_line_to (drop_path, theta, rho);
+				}
+			}
+			gog_renderer_stroke_serie (view->renderer, drop_path);
+			go_path_free (drop_path);
+			gog_renderer_pop_style (view->renderer);
+		}
+
+		gog_renderer_stroke_serie (view->renderer, path);
+		go_path_free (path);
+
 		if (is_polar)
-			c_vals = go_data_vector_get_values (GO_DATA_VECTOR (series->base.values[0].data));
-		for (count = 0; count < series->base.num_elements; count++) {
-
-			if (!gog_axis_map_finite (r_map, r_vals [count])) {
-				closed_shape = FALSE;
-				continue;
-			}
-			
-			path[count].code = ((count != 0 && !isnan (r_vals[count-1])) 
-					    ? ART_LINETO : ART_MOVETO);
-
-			rho = (!is_polar || (go_add_epsilon (r_vals[count] - rho_min) >= 0.0)) ?
-				r_vals[count] : rho_min;
-			gog_chart_map_2D_to_view (chart_map, 
-						  is_polar ? c_vals[count] : count + th0, rho,
-						  &path[count].x, &path[count].y);
-
-			if (is_polar) theta = gog_axis_map_to_view (c_map, c_vals[count]);
-
-			if ( !is_polar || 
-			     (go_add_epsilon (r_vals[count] - rho_min) >= 0.0 && 
-			      go_add_epsilon (rho_max - r_vals[count]) >= 0.0 &&
-			      go_add_epsilon ((theta_max - theta_min) - fmod (theta_max - theta, 2 * M_PI)) >= 0.0 &&
-			      go_add_epsilon ((theta_max - theta_min) - fmod (theta - theta_min, 2 * M_PI)) >= 0.0))
-				gog_renderer_draw_marker (view->renderer, path[count].x, path[count].y);
-		}
-
-		if (!is_polar && series->base.num_elements == model->num_elements
-		    && gog_axis_map_finite (r_map, r_vals[count-1])) {
-			path[count].code = ART_LINETO; 
-			path[count].x = path[0].x;
-			path[count].y = path[0].y;
-			count++;
-		}
-		path[count].code = ART_END;
-
-		if (is_polar) { 
-			bpath = gog_renderer_get_ring_wedge_bpath (parms->cx, parms->cy, parms->rx, parms->ry,
-								   0.0, 0.0, -parms->th0, -parms->th1);
-			if (bpath != NULL) {
-				clip_path = art_bez_path_to_vec (bpath, .1);
-				gog_renderer_push_clip (view->renderer, clip_path);	
-			}
-		}
-
-		if (closed_shape && is_area) {
-			gog_renderer_draw_polygon (view->renderer, path, FALSE);
-		} else
-			gog_renderer_draw_path (view->renderer, path);
-
-		if (is_polar && bpath != NULL) {
-			g_free (bpath);
 			gog_renderer_pop_clip (view->renderer);
+
+		if (gog_style_is_marker_visible (style)) {
+			for (count = 0; count < series->base.num_elements; count++) {
+				rho = (!is_polar || (go_add_epsilon (r_vals[count] - rho_min) >= 0.0)) ?
+					r_vals[count] : rho_min;
+				gog_chart_map_2D_to_view (chart_map, 
+							  is_polar ? c_vals[count] : count + 1, rho,
+							  &x, &y);
+
+				if (is_polar) theta = gog_axis_map_to_view (c_map, c_vals[count]);
+
+				if ( !is_polar || 
+				     (go_add_epsilon (r_vals[count] - rho_min) >= 0.0 && 
+				      go_add_epsilon (rho_max - r_vals[count]) >= 0.0 &&
+				      go_add_epsilon ((theta_max - theta_min) 
+						      - fmod (theta_max - theta, 2 * M_PI)) >= 0.0 &&
+				      go_add_epsilon ((theta_max - theta_min) 
+						      - fmod (theta - theta_min, 2 * M_PI)) >= 0.0))
+					gog_renderer_draw_marker (view->renderer, x, y);
+			}
 		}
 
 		gog_renderer_pop_style (view->renderer);
 	}
 	gog_chart_map_free (chart_map);
+
+	if (next_path != NULL)
+		go_path_free (next_path);
 }
 
 static void
@@ -525,6 +618,31 @@ GSF_DYNAMIC_CLASS (GogRTView, gog_rt_view,
 
 /*****************************************************************************/
 
+static gboolean
+radial_drop_lines_can_add (GogObject const *parent)
+{
+	GogRTSeries *series = GOG_RT_SERIES (parent);
+	return (   series->radial_drop_lines == NULL 
+		&& GOG_IS_PLOT_POLAR (gog_series_get_plot (GOG_SERIES (parent))));
+}
+
+static void
+radial_drop_lines_post_add (GogObject *parent, GogObject *child)
+{
+	GogRTSeries *series = GOG_RT_SERIES (parent);
+	series->radial_drop_lines = child;
+	gog_object_request_update (child);
+}
+
+static void
+radial_drop_lines_pre_remove (GogObject *parent, GogObject *child)
+{
+	GogRTSeries *series = GOG_RT_SERIES (parent);
+	series->radial_drop_lines = NULL;
+}
+
+/*****************************************************************************/
+
 static GogStyledObjectClass *series_parent_klass;
 
 static void
@@ -554,6 +672,16 @@ gog_rt_series_update (GogObject *obj)
 		((GogObjectClass *)series_parent_klass)->update(obj);
 }
 
+static void
+gog_rt_series_init (GObject *obj)
+{
+	GogSeries *series = GOG_SERIES (obj);
+	GogRTSeries *rt_series = GOG_RT_SERIES (obj);
+
+	series->fill_type = GOG_SERIES_FILL_TYPE_ORIGIN;
+	rt_series->radial_drop_lines = NULL;
+}
+
 static void
 gog_rt_series_init_style (GogStyledObject *gso, GogStyle *style)
 {
@@ -566,25 +694,65 @@ gog_rt_series_init_style (GogStyledObject *gso, GogStyle *style)
 
 	plot = GOG_RT_PLOT (series->plot);
 
-	if (!plot->default_style_has_markers &&
-	    style->marker.auto_shape) 
+	if (!plot->default_style_has_markers && style->marker.auto_shape)
 		go_marker_set_shape (style->marker.mark, GO_MARKER_NONE);
+
+	if (!plot->default_style_has_fill && style->fill.auto_type)
+		style->fill.type = GOG_FILL_STYLE_NONE;
 }
 
 static void
 gog_rt_series_class_init (GogStyledObjectClass *gso_klass)
 {
-	GogObjectClass * obj_klass = (GogObjectClass *) gso_klass;
+	static GogObjectRole const roles[] = {
+		{ N_("Radial drop lines"), "GogSeriesLines", 2,
+			GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
+			radial_drop_lines_can_add,
+			NULL,
+			NULL,
+			radial_drop_lines_post_add,
+			radial_drop_lines_pre_remove,
+			NULL }
+	};
 
-	series_parent_klass = g_type_class_peek_parent (gso_klass);
-	gso_klass->init_style = gog_rt_series_init_style;
+	GogObjectClass *obj_klass = GOG_OBJECT_CLASS (gso_klass);
+	GogSeriesClass *series_klass = GOG_SERIES_CLASS (gso_klass);
+
+	series_parent_klass = 	g_type_class_peek_parent (gso_klass);
+	gso_klass->init_style =	gog_rt_series_init_style;
 	obj_klass->update = gog_rt_series_update;
+
+	series_klass->has_interpolation = 	TRUE;
+
+	gog_object_register_roles (obj_klass, roles, G_N_ELEMENTS (roles));
 }
 
 GSF_DYNAMIC_CLASS (GogRTSeries, gog_rt_series,
-	gog_rt_series_class_init, NULL,
+	gog_rt_series_class_init, gog_rt_series_init,
 	GOG_SERIES_TYPE)
 
+static void
+gog_polar_series_class_init (GogObjectClass *gog_klass)
+{
+	static GogSeriesFillType const valid_fill_type_list[] = {
+		GOG_SERIES_FILL_TYPE_CENTER,
+		GOG_SERIES_FILL_TYPE_EDGE,
+		GOG_SERIES_FILL_TYPE_ORIGIN,
+		GOG_SERIES_FILL_TYPE_SELF,
+		GOG_SERIES_FILL_TYPE_NEXT,
+		GOG_SERIES_FILL_TYPE_INVALID
+	};
+
+	GogSeriesClass *series_klass = GOG_SERIES_CLASS (gog_klass);
+
+	series_klass->has_fill_type =		TRUE;
+	series_klass->valid_fill_type_list = 	valid_fill_type_list;
+}
+
+GSF_DYNAMIC_CLASS (GogPolarSeries, gog_polar_series,
+	gog_polar_series_class_init, NULL,
+	GOG_RT_SERIES_TYPE)
+
 G_MODULE_EXPORT void
 go_plugin_init (GOPlugin *plugin, GOCmdContext *cc)
 {
@@ -595,6 +763,7 @@ go_plugin_init (GOPlugin *plugin, GOCmdContext *cc)
 	gog_polar_plot_register_type (module);
 	gog_rt_view_register_type (module);
 	gog_rt_series_register_type (module);
+	gog_polar_series_register_type (module);
 }
 
 G_MODULE_EXPORT void
